@@ -49,9 +49,6 @@ func TestInjectSidecarOpenShift(t *testing.T) {
 	defer reset()
 
 	jaeger := v1.NewJaeger(types.NamespacedName{Name: "my-instance"})
-	assert.Len(t, jaeger.Spec.Agent.VolumeMounts, 0)
-	assert.Len(t, jaeger.Spec.Agent.Volumes, 0)
-
 	dep := dep(map[string]string{}, map[string]string{})
 	dep = Sidecar(jaeger, dep)
 	assert.Equal(t, dep.Labels[Label], jaeger.Name)
@@ -60,10 +57,6 @@ func TestInjectSidecarOpenShift(t *testing.T) {
 	assert.Len(t, dep.Spec.Template.Spec.Containers[0].Env, 0)
 	assert.Len(t, dep.Spec.Template.Spec.Containers[1].VolumeMounts, 2)
 	assert.Len(t, dep.Spec.Template.Spec.Volumes, 2)
-
-	// CR should not be touched.
-	assert.Len(t, jaeger.Spec.Agent.VolumeMounts, 0)
-	assert.Len(t, jaeger.Spec.Agent.Volumes, 0)
 }
 
 func TestInjectSidecarWithEnvVars(t *testing.T) {
@@ -81,7 +74,7 @@ func TestInjectSidecarWithEnvVars(t *testing.T) {
 	containsEnvVarNamed(t, dep.Spec.Template.Spec.Containers[1].Env, envVarHostIP)
 
 	assert.Len(t, dep.Spec.Template.Spec.Containers[0].Env, 2)
-	assert.Contains(t, dep.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{Name: envVarPropagation, Value: "jaeger,b3"})
+	assert.Contains(t, dep.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{Name: envVarPropagation, Value: "jaeger,b3,w3c"})
 	assert.Contains(t, dep.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{Name: envVarServiceName, Value: "testapp.default"})
 }
 
@@ -135,7 +128,7 @@ func TestInjectSidecarWithEnvVarsWithNamespace(t *testing.T) {
 
 	assert.Len(t, dep.Spec.Template.Spec.Containers[0].Env, 2)
 	assert.Contains(t, dep.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{Name: envVarServiceName, Value: "testapp.mynamespace"})
-	assert.Contains(t, dep.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{Name: envVarPropagation, Value: "jaeger,b3"})
+	assert.Contains(t, dep.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{Name: envVarPropagation, Value: "jaeger,b3,w3c"})
 }
 
 func TestInjectSidecarWithEnvVarsOverrideName(t *testing.T) {
@@ -157,7 +150,7 @@ func TestInjectSidecarWithEnvVarsOverrideName(t *testing.T) {
 
 	assert.Len(t, dep.Spec.Template.Spec.Containers[0].Env, 2)
 	assert.Contains(t, dep.Spec.Template.Spec.Containers[0].Env, envVar)
-	assert.Contains(t, dep.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{Name: envVarPropagation, Value: "jaeger,b3"})
+	assert.Contains(t, dep.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{Name: envVarPropagation, Value: "jaeger,b3,w3c"})
 }
 
 func TestInjectSidecarWithEnvVarsOverridePropagation(t *testing.T) {
@@ -182,6 +175,77 @@ func TestInjectSidecarWithEnvVarsOverridePropagation(t *testing.T) {
 	assert.Contains(t, dep.Spec.Template.Spec.Containers[0].Env, corev1.EnvVar{Name: envVarServiceName, Value: "testapp.default"})
 }
 
+func TestInjectSidecarWithVolumeMounts(t *testing.T) {
+	// prepare
+	jaeger := v1.NewJaeger(types.NamespacedName{Name: "my-instance"})
+	dep := dep(map[string]string{}, map[string]string{})
+
+	agentVolume := corev1.Volume{
+		Name: "test-volume1",
+		VolumeSource: corev1.VolumeSource{
+			Secret: &corev1.SecretVolumeSource{
+				SecretName: "test-secret1",
+			},
+		},
+	}
+	agentVolumeMount := corev1.VolumeMount{
+		Name:      "test-volume1",
+		MountPath: "/test-volume1",
+		ReadOnly:  true,
+	}
+
+	commonVolume := corev1.Volume{
+		Name: "test-volume2",
+		VolumeSource: corev1.VolumeSource{
+			Secret: &corev1.SecretVolumeSource{
+				SecretName: "test-secret2",
+			},
+		},
+	}
+	commonVolumeMount := corev1.VolumeMount{
+		Name:      "test-volume2",
+		MountPath: "/test-volume2",
+		ReadOnly:  true,
+	}
+
+	jaeger.Spec.Agent.Volumes = append(jaeger.Spec.Agent.Volumes, agentVolume)
+	jaeger.Spec.Agent.VolumeMounts = append(jaeger.Spec.Agent.VolumeMounts, agentVolumeMount)
+	jaeger.Spec.Volumes = append(jaeger.Spec.Volumes, commonVolume)
+	jaeger.Spec.VolumeMounts = append(jaeger.Spec.VolumeMounts, commonVolumeMount)
+
+	// test
+	dep = Sidecar(jaeger, dep)
+
+	// verify
+	assert.Contains(t, dep.Spec.Template.Spec.Volumes, agentVolume)
+	assert.NotContains(t, dep.Spec.Template.Spec.Volumes, commonVolume)
+	assert.Contains(t, dep.Spec.Template.Spec.Containers[1].Image, "jaeger-agent")
+	assert.Contains(t, dep.Spec.Template.Spec.Containers[1].VolumeMounts, agentVolumeMount)
+	assert.NotContains(t, dep.Spec.Template.Spec.Containers[1].VolumeMounts, commonVolumeMount)
+}
+
+func TestSidecarImagePullSecrets(t *testing.T) {
+
+	deploymentImagePullSecrets := []corev1.LocalObjectReference{{
+		Name: "deploymentImagePullSecret",
+	}}
+
+	agentImagePullSecrets := []corev1.LocalObjectReference{{
+		Name: "agentImagePullSecret",
+	}}
+
+	jaeger := v1.NewJaeger(types.NamespacedName{Name: "my-instance"})
+	jaeger.Spec.Agent.ImagePullSecrets = agentImagePullSecrets
+
+	dep := dep(map[string]string{}, map[string]string{})
+	dep.Spec.Template.Spec.ImagePullSecrets = deploymentImagePullSecrets
+	dep = Sidecar(jaeger, dep)
+
+	assert.Len(t, dep.Spec.Template.Spec.ImagePullSecrets, 2)
+	assert.Equal(t, dep.Spec.Template.Spec.ImagePullSecrets[0].Name, "deploymentImagePullSecret")
+	assert.Equal(t, dep.Spec.Template.Spec.ImagePullSecrets[1].Name, "agentImagePullSecret")
+}
+
 func TestSidecarDefaultPorts(t *testing.T) {
 	// prepare
 	jaeger := v1.NewJaeger(types.NamespacedName{Name: "my-instance"})
@@ -216,8 +280,17 @@ func TestSkipInjectSidecar(t *testing.T) {
 }
 
 func TestSidecarNeeded(t *testing.T) {
-	depWithAgent := dep(map[string]string{}, map[string]string{})
-	depWithAgent.Spec.Template.Spec.Containers = append(depWithAgent.Spec.Template.Spec.Containers, corev1.Container{
+
+	jaeger := v1.NewJaeger(types.NamespacedName{Name: "some-jaeger-instance"})
+
+	depWithAgent := dep(map[string]string{
+		Annotation: "some-jaeger-instance",
+	}, map[string]string{})
+
+	depWithAgent = Sidecar(jaeger, depWithAgent)
+
+	explicitInjected := dep(map[string]string{}, map[string]string{})
+	explicitInjected.Spec.Template.Spec.Containers = append(explicitInjected.Spec.Template.Spec.Containers, corev1.Container{
 		Name: "jaeger-agent",
 	})
 
@@ -249,17 +322,28 @@ func TestSidecarNeeded(t *testing.T) {
 		{
 			dep:    depWithAgent,
 			ns:     ns(map[string]string{}),
-			needed: false,
+			needed: true,
 		},
 		{
 			dep:    dep(map[string]string{}, map[string]string{"app": "jaeger"}),
 			ns:     ns(map[string]string{Annotation: "true"}),
 			needed: false,
 		},
+		{
+			dep:    explicitInjected,
+			ns:     ns(map[string]string{}),
+			needed: false,
+		},
+		{
+			dep:    explicitInjected,
+			ns:     ns(map[string]string{Annotation: "true"}),
+			needed: false,
+		},
 	}
 	for _, test := range tests {
 		t.Run(fmt.Sprintf("dep:%s, ns: %s", test.dep.Annotations, test.ns.Annotations), func(t *testing.T) {
 			assert.Equal(t, test.needed, Needed(test.dep, test.ns))
+			assert.LessOrEqual(t, len(test.dep.Spec.Template.Spec.Containers), 2)
 		})
 	}
 }
@@ -267,6 +351,13 @@ func TestSidecarNeeded(t *testing.T) {
 func TestSelect(t *testing.T) {
 	jTest := v1.NewJaeger(types.NamespacedName{Name: "test"})
 	jProd := v1.NewJaeger(types.NamespacedName{Name: "prod"})
+
+	depNsProd := dep(map[string]string{Annotation: "true"}, map[string]string{})
+	depNsProd.Namespace = "nsprod"
+
+	jTestNsTest := v1.NewJaeger(types.NamespacedName{Name: "test", Namespace: "nstest"})
+	jProdNsProd := v1.NewJaeger(types.NamespacedName{Name: "prod", Namespace: "nsprod"})
+
 	tests := []struct {
 		dep      *appsv1.Deployment
 		ns       *corev1.Namespace
@@ -351,6 +442,20 @@ func TestSelect(t *testing.T) {
 			expected: nil,
 			cap:      "dep none, ns true, no jaegers",
 		},
+		{
+			dep:      depNsProd,
+			ns:       ns(map[string]string{}),
+			jaegers:  &v1.JaegerList{Items: []v1.Jaeger{*jTestNsTest, *jProdNsProd}},
+			expected: jProdNsProd,
+			cap:      "dep true, two jaeger instances one in the same ns",
+		},
+		{
+			dep:      depNsProd,
+			ns:       ns(map[string]string{Annotation: "true"}),
+			jaegers:  &v1.JaegerList{Items: []v1.Jaeger{*jTestNsTest, *jProdNsProd}},
+			expected: jProdNsProd,
+			cap:      "dep none, ns true, two jaeger instances one in the same ns",
+		},
 	}
 
 	for _, test := range tests {
@@ -397,13 +502,12 @@ func TestSidecarOrderOfArguments(t *testing.T) {
 	dep = Sidecar(jaeger, dep)
 
 	assert.Len(t, dep.Spec.Template.Spec.Containers, 2)
-	assert.Len(t, dep.Spec.Template.Spec.Containers[1].Args, 6)
+	assert.Len(t, dep.Spec.Template.Spec.Containers[1].Args, 5)
 	containsOptionWithPrefix(t, dep.Spec.Template.Spec.Containers[1].Args, "--a-option")
 	containsOptionWithPrefix(t, dep.Spec.Template.Spec.Containers[1].Args, "--b-option")
 	containsOptionWithPrefix(t, dep.Spec.Template.Spec.Containers[1].Args, "--c-option")
 	containsOptionWithPrefix(t, dep.Spec.Template.Spec.Containers[1].Args, "--jaeger.tags")
 	containsOptionWithPrefix(t, dep.Spec.Template.Spec.Containers[1].Args, "--reporter.grpc.host-port")
-	containsOptionWithPrefix(t, dep.Spec.Template.Spec.Containers[1].Args, "--reporter.type")
 	agentTags := agentTags(dep.Spec.Template.Spec.Containers[1].Args)
 	assert.Contains(t, agentTags, "container.name=only_container")
 }
@@ -423,20 +527,18 @@ func TestSidecarExplicitTags(t *testing.T) {
 	assert.Equal(t, []string{"key=val"}, agentTags)
 }
 
-func TestSidecarOverrideReporter(t *testing.T) {
+func TestSidecarCustomReporterPort(t *testing.T) {
 	jaeger := v1.NewJaeger(types.NamespacedName{Name: "my-instance"})
 	jaeger.Spec.Agent.Options = v1.NewOptions(map[string]interface{}{
-		"reporter.type":             "thrift",
-		"reporter.thrift.host-port": "collector:14267",
+		"reporter.grpc.host-port": "collector:5000",
 	})
 
 	dep := dep(map[string]string{}, map[string]string{})
 	dep = Sidecar(jaeger, dep)
 
 	assert.Len(t, dep.Spec.Template.Spec.Containers, 2)
-	assert.Len(t, dep.Spec.Template.Spec.Containers[1].Args, 3)
-	assert.Contains(t, dep.Spec.Template.Spec.Containers[1].Args, "--reporter.thrift.host-port=collector:14267")
-	assert.Contains(t, dep.Spec.Template.Spec.Containers[1].Args, "--reporter.type=thrift")
+	assert.Len(t, dep.Spec.Template.Spec.Containers[1].Args, 2)
+	assert.Contains(t, dep.Spec.Template.Spec.Containers[1].Args, "--reporter.grpc.host-port=collector:5000")
 }
 
 func TestSidecarAgentResources(t *testing.T) {
@@ -657,10 +759,9 @@ func TestSidecarArgumentsOpenshiftTLS(t *testing.T) {
 	dep = Sidecar(jaeger, dep)
 
 	assert.Len(t, dep.Spec.Template.Spec.Containers, 2)
-	assert.Len(t, dep.Spec.Template.Spec.Containers[1].Args, 6)
+	assert.Len(t, dep.Spec.Template.Spec.Containers[1].Args, 5)
 	assert.Greater(t, len(util.FindItem("--a-option=a-value", dep.Spec.Template.Spec.Containers[1].Args)), 0)
 	assert.Greater(t, len(util.FindItem("--jaeger.tags", dep.Spec.Template.Spec.Containers[1].Args)), 0)
-	assert.Greater(t, len(util.FindItem("--reporter.type=grpc", dep.Spec.Template.Spec.Containers[1].Args)), 0)
 	assert.Greater(t, len(util.FindItem("--reporter.grpc.host-port=dns:///my-instance-collector-headless.test.svc:14250", dep.Spec.Template.Spec.Containers[1].Args)), 0)
 	assert.Greater(t, len(util.FindItem("--reporter.grpc.tls.enabled=true", dep.Spec.Template.Spec.Containers[1].Args)), 0)
 	assert.Greater(t, len(util.FindItem("--reporter.grpc.tls.ca="+ca.ServiceCAPath, dep.Spec.Template.Spec.Containers[1].Args)), 0)
@@ -668,6 +769,75 @@ func TestSidecarArgumentsOpenshiftTLS(t *testing.T) {
 	assert.Contains(t, agentTags, "container.name=only_container")
 }
 
+func TestEqualSidecar(t *testing.T) {
+	jaeger := v1.NewJaeger(types.NamespacedName{
+		Name:      "my-instance",
+		Namespace: "test",
+	})
+
+	dep1 := dep(map[string]string{Annotation: jaeger.Name}, map[string]string{})
+	dep1 = Sidecar(jaeger, dep1)
+
+	dep1Equal := dep(map[string]string{Annotation: jaeger.Name}, map[string]string{})
+	dep1Equal = Sidecar(jaeger, dep1Equal)
+	assert.True(t, EqualSidecar(dep1, dep1Equal))
+
+	// Change flags.
+	jaeger.Spec.Agent.Options = v1.NewOptions(map[string]interface{}{
+		"--jaeger.tags": "changed-tag=newvalue",
+	})
+
+	dep2 := dep(map[string]string{Annotation: jaeger.Name}, map[string]string{})
+	dep2 = Sidecar(jaeger, dep2)
+	assert.False(t, EqualSidecar(dep1, dep2))
+
+	// When no agent is present on the deploy
+	dep3 := dep(map[string]string{Annotation: jaeger.Name}, map[string]string{})
+	assert.False(t, EqualSidecar(dep1, dep3))
+}
+
+func TestAgentOTELConfig(t *testing.T) {
+	jaeger := v1.NewJaeger(types.NamespacedName{Name: "my-instance"})
+	jaeger.Spec.Agent.Config = v1.NewFreeForm(map[string]interface{}{"foo": "bar"})
+
+	d := Sidecar(jaeger, &appsv1.Deployment{
+		ObjectMeta: metav1.ObjectMeta{
+			Labels:      map[string]string{Label: "my-instance"},
+			Annotations: map[string]string{},
+		},
+	})
+	assert.True(t, hasArgument("--config=/etc/jaeger/otel/config.yaml", d.Spec.Template.Spec.Containers[0].Args))
+	assert.True(t, hasVolume("my-instance-agent-otel-config", d.Spec.Template.Spec.Volumes))
+	assert.True(t, hasVolumeMount("my-instance-agent-otel-config", d.Spec.Template.Spec.Containers[0].VolumeMounts))
+}
+
+func hasVolume(name string, volumes []corev1.Volume) bool {
+	for _, v := range volumes {
+		if v.Name == name {
+			return true
+		}
+	}
+	return false
+}
+
+func hasVolumeMount(name string, volumeMounts []corev1.VolumeMount) bool {
+	for _, v := range volumeMounts {
+		if v.Name == name {
+			return true
+		}
+	}
+	return false
+}
+
+func hasArgument(arg string, args []string) bool {
+	for _, v := range args {
+		if v == arg {
+			return true
+		}
+	}
+	return false
+}
+
 func TestInjectSidecarOnOpenShift(t *testing.T) {
 	viper.Set("platform", v1.FlagPlatformOpenShift)
 	defer viper.Reset()
@@ -680,3 +850,16 @@ func TestInjectSidecarOnOpenShift(t *testing.T) {
 	assert.Len(t, dep.Spec.Template.Spec.Containers[1].VolumeMounts, 2)
 	assert.Len(t, dep.Spec.Template.Spec.Volumes, 2)
 }
+
+func TestSidecarWithSecurityContext(t *testing.T) {
+	var user, group int64 = 111, 222
+	expectedSecurityContext := &corev1.SecurityContext{RunAsUser: &user, RunAsGroup: &group}
+
+	jaeger := v1.NewJaeger(types.NamespacedName{Name: "TestSidecarWithSecurityContext"})
+	jaeger.Spec.Agent.SidecarSecurityContext = expectedSecurityContext
+
+	dep := dep(map[string]string{}, map[string]string{})
+	dep = Sidecar(jaeger, dep)
+	assert.Len(t, dep.Spec.Template.Spec.Containers, 2)
+	assert.Equal(t, dep.Spec.Template.Spec.Containers[1].SecurityContext, expectedSecurityContext)
+}
